##############################################################
### Estimating gPDC at the RSA frequency band for one dyad ###
##############################################################


### Step 1: Define function gPDC for estimating gPDC from outputs of a VAR model (Eqs. 2 & 3)
gPDC = function(AR,       # AR coefficients from VAR
                ny,       # number of variables in the time series 
                ORDER,    # order of VAR
                ECOV,     # error covariance from VAR
                nfreq,    # number of discrete frequencies
                nwin)     # number of time points
{
  
  #### defining matrix dimensions
  qf <- rep(NA, nfreq)
  q <- array(NA,dim=c(ORDER, nfreq))
  # I-M=AF
  M <- array(NA,dim=c(ny,ny,nfreq))
  AF <- array(NA,dim=c(ny,ny,nfreq))
  I <- diag(ny)
  # Hermitian transpose of A(f)
  AFH <- array(NA,dim=c(ny,ny,nfreq))
  
  gPDC <- array(NA,dim=c(ny,ny,nfreq))
  gABI <- array(NA,dim=c(ny,ny,nfreq))
  gSUMI <- array(0, dim=c(ny, nfreq))
  
  # compute qf and q
  for (f in 1:nfreq) {
    qf[f] <- f*2*pi/nwin
    for (k in 1:ORDER) {
      q[k,f] <- qf[f]*k
    }}
  
  for (i in 1:ny) {
    for (j in 1:ny) {
      # M is the intermidiate matrix. I-M=AF
      M[i,j,] <- complex(real= AR[i,j,]%*%cos(q), im= AR[i,j,]%*%sin(q))
    }}
  
  for (f in 1:nfreq) {
    for (i in 1:ny) {
      for (j in 1:ny) {
        # Computing A(f)
        AF[i,j,f] <- I[i,j] - M[i,j,f]
      }}}
  
  # Squares of denominators for gPDC
  for (f in 1:nfreq) {
    for (j in 1:ny) {
      for (i in 1:ny) {
        gABI[i,j,f] <- ((abs(AF[i,j,f]))^2)/ECOV[i,i] 
        gSUMI[j,f] <- gSUMI[j,f]+gABI[i,j,f] 
      }}}
  
  
  for (f in 1:nfreq) {
    for (i in 1:ny) {
      for (j in 1:ny) {
        # Computing gPDC
       gPDC[i,j,f] <- (abs(AF[i,j,f]) / sqrt(ECOV[i,i])) / sqrt(gSUMI[j,f]) # computed from VAR, scaled by error variance
    }}}
  
  # Output
  OUT <- list(gPDC=gPDC)
  return(OUT)
}

# End of function gPDC

### Step 2: Read in, difference, and center time series IBI data
### Note: Data should be a nt*ny matrix, where nt is the number of time points
### and ny is the number of variables (e.g., persons)
DATA <- as.matrix(read.table("https://4973696-464392845695117242.preview.editmysite.com/uploads/4/9/7/3/4973696/dyad.txt"),header=FALSE)

# difference the original data
DATAD <- diff(DATA) 

# extract dimensions of differenced data set for later use
ny <- ncol(DATAD)  
nt <- nrow(DATAD)

# Examine whether number of time points is even. If not, delete the last time point to make it even.
is.even <- function(x) x %% 2 == 0 
if (is.even(nt) == TRUE) {
  nwin <- nt}
if (is.even(nt) == FALSE) {
  nwin <- nt-1
}

# Center data to get rid of zero frequency
DATAC <- array(NA,dim=c(nwin, ny)) 
for (c in 1:ny) {
  DATAC[,c] <- scale(DATAD[1:nwin,c], center=TRUE, scale=FALSE) 
}


### Step 3: Fit VAR to centered data
VARout <- ar(DATAC,aic=TRUE,demean="FALSE") 

### Step 4: Extract VAR outputs and estimate gPDC using function gPDC (defined in Step 1)
# order of VAR
order<- VARout$order

# AR matrices of VAR
B <- array(NA,dim=c(ny,ny,order))
for (p in 1:order){
  for (i in 1:ny){
    for (j in 1:ny){
      B[i,j,p] <- VARout$ar[p,i,j]
    }}}

# error covariance matrix of VAR
EV <- array(NA,dim=c(ny,ny))
EV <- VARout$var.pred

# number of discrete frequencies
nfreq <- nwin/2

# Estimate gPDC using function defined in Step 1
gPDCout <- gPDC(B,ny,order,EV,nfreq,nwin)   

# Output gPDC
gPDC12 <- gPDCout$gPDC[1,2,1:nfreq]  # gPDC from variable 2 to variable 1
gPDC21 <- gPDCout$gPDC[2,1,1:nfreq]  # gPDC from variable 1 to variable 2
# Note: For higher-dimentional data, gPDC can be outputed in a similar way


### Step 5: Compute average gPDC for the RSA frequency band
# Input sampling rate in Hz, e.g., sr <- 4 if sampling rate is 4 Hz
sr <- 4
# Input lower bound of RSA frequency range, e.g., 0.12 for adults
lb <- 0.12
# Input higher bound of RSA frequency range, e.g., 0.40 for adults
hb <- 0.4
# Compute lower and higher bounds for RSA frequency band
lf = round(lb*(nwin-1)/sr)
hf = round(hb*(nwin-1)/sr)
  
# Average across RSA frequencies 
gPDC12_RSAband <- mean(gPDC12[lf:hf]) # Influence from variable 2 to variable 1 at RSA frequency band
gPDC21_RSAband <- mean(gPDC21[lf:hf]) # Influence from variable 1 to variable 2 at RSA frequency band

gPDC12_RSAband #0.1060422
gPDC21_RSAband #0.08268305
    
